using System.Diagnostics;
using ttrss_co_client.ttrss;
using ttrss_co_client.ttrss.datastructures;

namespace ttrss_co_client.tasks
{
    ///<summary>download from YT</summary>
    public class PodcastifyYT : Phase1Task
    {
        public override string TaskName => "podcastifyYT";
        public override async Task<WorkOrder> ActOn(Headline headline, IEnumerable<Label> labelsWRTArticle)
        {
            Console.WriteLine($"    YT podcastify: {headline.link.ToString()}");
            var myGuid = Guid.NewGuid().ToString();
            var ytdl = new YoutubeDLSharp.YoutubeDL();
            ytdl.YoutubeDLPath = "yt-dlp";
            ytdl.FFmpegPath = "ffmpeg";
            ytdl.OutputFolder = $"{Conf.WorkingDirectory}/{myGuid}";
            ytdl.OutputFileTemplate = "%(upload_date)s - %(title)s - [%(id)s].%(ext)s";
            var sw = new Stopwatch();
            if(!Path.Exists(ytdl.OutputFolder))
            {
                Directory.CreateDirectory(ytdl.OutputFolder);
            }

            try
            {
                sw.Start();
                var res = await ytdl.RunVideoDownload(headline.link.ToString());
                sw.Stop();

                var outputStr = $"download {(res.Success ? "success" : "fail")} in {sw.Elapsed}";
                if (res.ErrorOutput != null && res.ErrorOutput.Length > 0)
                {
                    outputStr += "\n" + string.Join('\n', res.ErrorOutput);
                }
                Console.WriteLine($"    {headline.link} -> {res.Data}\n{outputStr}");
                await TtrssClient.SetArticleLabel(labelsWRTArticle.First(l => l.caption?.ToLower() == this.TriggerLabel.ToLower()).id, false, headline.id);
                Console.WriteLine($"    {headline.title}: download trigger label removed");

                var podcastTitle = headline.feed_title;
                var titlingLabel = labelsWRTArticle.FirstOrDefault(l => l.@checked && l.caption?.ToLower().StartsWith(Conf.PodcastTitlePrefix) == true);
                if(titlingLabel != null)
                {
                    await TtrssClient.SetArticleLabel(titlingLabel.id, false, headline.id);
                    podcastTitle = titlingLabel.caption.Substring(Conf.PodcastTitlePrefix.Length);
                }
                Console.WriteLine($"article {headline.id} - podcastifying; {podcastTitle}");
                
                await TtrssClient.UpdateArticleNote($"{headline.note}\n[{DateTime.Now.ToString("o")}] - YT podcastify (dl) {(res.Success ? "success" : "failure")}; {string.Join('\n', res.ErrorOutput)}", headline.id);
                if (!res.Success)
                {
                    return null;
                }
                else
                {
                    var outputFilename = res.Data;
                    Console.WriteLine($"{headline.title} downloaded.");

                    var toReturn = new WorkOrder()
                    {
                        articleId = headline.id,//<-- that way later tasks can update the note
                        Phase2TaskList = new Dictionary<int, string>(),
                        data = new Dictionary<string, string>(),
                        guid = Guid.Parse(myGuid)
                    };
                    toReturn.data["path"] = outputFilename;
                    if(!outputFilename.EndsWith(".mp3"))
                    {
                        Console.WriteLine($"{headline.title} needs conversion task.");
                        toReturn.Phase2TaskList[0] = "convert";
                        toReturn.data["conversion-target"] = outputFilename.Substring(0, res.Data.LastIndexOf('.')) + ".mp3";
                    }

                    toReturn.Phase2TaskList[1] = "sponsorblock";
                    toReturn.Phase2TaskList[2] = "filemovePublish";
                    toReturn.data["publish-target"] = $"{Conf.OnDoneCopy}/Podcasts/{podcastTitle}/{Path.GetFileName(toReturn.data["conversion-target"])}";

                    return toReturn;
                }
            }
            catch (Exception e)
            {
                Console.Error.WriteLine($"fatal error in standard DL for {headline.link}");
                Console.Error.WriteLine($"{e.ToString()}: {e.Message}.\n{e.StackTrace}");
                await TtrssClient.UpdateArticleNote($"{headline.note}\n[{DateTime.Now.ToString("o")}] - fatal error {e.ToString()}: {e.Message}.\n{e.StackTrace}", headline.id);
                return null;
            }
        }
    }
}